local LB = LibStub:GetLibrary("LibBoss-1.0")
local ATLAS = ZO_Object:Subclass()

ATLAS.addonName = "Atlas"
ATLAS.defaults = { }

local ATLAS_LOCATION_DATA = 1
local ATLAS_DROPS_BOSS_DATA = 2
local ATLAS_DROPS_DROP_DATA = 3
local ATLAS_DROPS_SET_ITEM_DATA = 4
local ATLAS_LEVEL2_LABEL = 5

ZO_CreateStringId("ATLAS_LOOT_NAME", "ATLAS LOOT")

local ATLAS_ScrollList_SORT_KEYS =
{
    ["locationName"] = { },
    ["factionIcon"] = {  tiebreaker = "locationName" },
    ["NormalLevel"] = {  tiebreaker = "locationName" },
    --["VeteranLevel"] = {  tiebreaker = "locationName" },
}
--
-- list of control object references
--
local ATLAS_TLW = nil
local ATLAS_LootWindow = nil
local ATLAS_Map = nil
local ATLAS_RIGHTPANE = nil
--
-- more constants
--
local ATLAS_MAP_SCALE = 0.8

local DAGGERFALL = "D"
local EBONHEART  = "E"
local ALDMERI    = "A"
local UNKNOWN	 = "?"
--
-- Some variables that should be saved ...
--
ATLAS.isVeteranDifficulty = true
ATLAS.lastZoneRequested = nil
ATLAS.lastDungeonTypeRequested = nil
ATLAS.chatWasMinimized = false

local ATLAS_MAP_TEXTURES_IN_USE = -1
local ATLAS_MAP_PINS_IN_USE = -1
local ATLAS_MAX_DUNGEON_LEVELS = 5
local ATLAS_MAX_SET_TOOLTIPS_IN_USE = -1

local function AtlasToolTipCreator(control, toolTipText)
	control:SetMouseEnabled(true)

  	control:SetHandler("OnMouseEnter", function() 
  		InitializeTooltip(InformationTooltip, control, BOTTOM, 0, 0)
  		InformationTooltip:AddLine(toolTipText, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
  		InformationTooltip:SetHidden(false) 
  	end )

  	control:SetHandler("OnMouseExit", function() 
  		InformationTooltip:ClearLines()
  		InformationTooltip:SetHidden(true) 
  	end )
end

local function determineDataSource(atlasDungeonType, zoneName)	
	return LB:getZoneData(atlasDungeonType, zoneName)	
end

local function getItemID(itemLink)
	return tonumber(({ZO_LinkHandler_ParseLink(itemLink)})[4])
end

local function showAtlasLoot(zone, expandSet)
	local scrollData = ZO_ScrollList_GetDataList(ATLAS_LootWindow.scrollList)

	local level = nil -- Set to nil to get all 

	ZO_ClearNumericallyIndexedTable(scrollData)

	for _,bossData in ipairs(LB:returnBossList(zone,LB:determineZoneType(zone,ATLAS.isVeteranDifficulty),level)) do
		
		bossID = bossData[G_LIBBOSS.INDEX.BOSS_ID]
		table.insert(scrollData, ZO_ScrollList_CreateDataEntry(ATLAS_DROPS_BOSS_DATA, {bossName = bossData[G_LIBBOSS.INDEX.NAME]})) 

		local setName = nil
		for _, item in pairs(LB:returnBossLoot(bossID)) do
			if setName ~= LB:returnSetName(getItemID(item)) then
				setName = LB:returnSetName(getItemID(item))
				if setName then
					table.insert(scrollData, ZO_ScrollList_CreateDataEntry(ATLAS_LEVEL2_LABEL, {Label = zo_strformat("|cFF7F24<<1>>|r",setName)})) 
				end
			end
			
			if setName then
				table.insert(scrollData, ZO_ScrollList_CreateDataEntry(ATLAS_DROPS_SET_ITEM_DATA, {ItemLink = zo_strformat("<<1>>",item)})) 
			else
				table.insert(scrollData, ZO_ScrollList_CreateDataEntry(ATLAS_DROPS_DROP_DATA, {ItemLink = zo_strformat("<<1>>",item)})) 
			end
--			if expandSet and expandSet == item then
--				local quality = 0
--				local level = 0 
--				local index = 0
--				for value in string.gmatch(item, ":(%d+)") do
--					index = index + 1
--					if index == 1 then 
--						quality = value 
--					elseif index == 2 then 
--						level = value 
--					else
--						table.insert(scrollData, ZO_ScrollList_CreateDataEntry(ATLAS_DROPS_SET_ITEM_DATA, {ItemLink = zo_strformat("<<1>>",LB:getSetItemLink(value, quality, level))})) 
--					end 	
--				end
--			end
		end	

	end

	ZO_ScrollList_Commit(ATLAS_LootWindow.scrollList)
	ATLAS.lastZoneRequested = zone	
end

local function refreshAtlasLoot()
	if ATLAS.lastZoneRequested then		
		showAtlasLoot(ATLAS.lastZoneRequested)
	end
end

local function showAtlasWindows(toggle)	
	ATLAS_LootWindow:SetHidden( not toggle )
	ATLAS_Map:SetHidden( not toggle )
	ATLAS_TLW:SetHidden( not toggle )
	if not toggle then
		if not ATLAS.chatWasMinimized then
			CHAT_SYSTEM:Maximize()
		end
	end
end

local function createAtlasMapPins(parent, mapPinId, text, x, y)
	local mapPin = WINDOW_MANAGER:GetControlByName("ATLASMapPin"..mapPinId)
	if not mapPin then
		mapPin = WINDOW_MANAGER:CreateControl("ATLASMapPin"..mapPinId,  parent, CT_TEXTURE)
	end
  	mapPin:SetDimensions(32,32)
  	mapPin:SetTexture("/esoui/art/icons/poi/poi_groupboss_complete.dds")
  	mapPin:SetAnchor(CENTER, parent, TOPLEFT, x, y)  
  	mapPin:SetHidden(false)	
  	mapPin:SetMouseEnabled(true)

  	mapPin:SetHandler("OnMouseEnter", function() 
  			InitializeTooltip(InformationTooltip, mapPin, BOTTOM, 0, 0)
  			InformationTooltip:AddLine(text, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
  			InformationTooltip:SetHidden(false) 
  		end )
  	mapPin:SetHandler("OnMouseExit", function() 
  			InformationTooltip:ClearLines()
  			InformationTooltip:SetHidden(true) 
  		end )	
end

local function hideAtlasMapPin(mapPinId)
	WINDOW_MANAGER:GetControlByName("ATLASMapPin"..mapPinId):SetHidden(true)
end



local function buildAtlasMap(zone, atlasDungeonType, level)	
	ATLAS.lastZoneRequested = zone
	ATLAS.lastDungeonTypeRequested = atlasDungeonType
	
	SetMapToPlayerLocation()       
   	local mapName = GetMapName()
	if zone == mapName then
		ATLAS_Map.playerIcon:SetHidden(false)
	else
		ATLAS_Map.playerIcon:SetHidden(true)
	end

	if not atlasDungeonType then 
		atlasDungeonType = LB:determineZoneType(zone, ATLAS.isVeteranDifficulty )
	end

	if not atlasDungeonType then return end

	if not level then
		level = 1
	end

	local data = determineDataSource(atlasDungeonType, zone)

	if data == nil then return end
	--
	-- Blank out all previous textures
	--
	for i = 0, ATLAS_MAP_TEXTURES_IN_USE do		
		WINDOW_MANAGER:GetControlByName("AtlasMapTexture"..i):SetHidden(true)
	end
	--
	-- Rebuild the map textures
	--	
	local map = LB:getMapTextures(zone, atlasDungeonType, level)
	
	local x,y = ATLAS_Map:GetDimensions()

	if map then
		showAtlasWindows(true)

		for i = 0 , (map[G_LIBBOSS.TEXTURES.X]*map[G_LIBBOSS.TEXTURES.Y]) - 1 do			
			local name = "AtlasMapTexture"..i
			local texture = WINDOW_MANAGER:GetControlByName(name)

			if not texture then
				texture = WINDOW_MANAGER:CreateControl(name, ATLAS_Map.bd, CT_TEXTURE)
			end
		
			texture:SetDimensions(x/map[G_LIBBOSS.TEXTURES.X],y/map[G_LIBBOSS.TEXTURES.Y])  		
			texture:SetAnchor( TOPLEFT, ATLAS_Map, TOPLEFT, (x/map[G_LIBBOSS.TEXTURES.X])*(math.fmod(i,map[G_LIBBOSS.TEXTURES.X])), (y/map[G_LIBBOSS.TEXTURES.Y])*(math.floor(i/map[G_LIBBOSS.TEXTURES.Y])) )
			texture:SetTexture(map[G_LIBBOSS.TEXTURES.PATH]..i..".dds") 							
			texture:SetHidden( false )			
			texture:SetDrawLayer(0)
			
		end
		if ATLAS_MAP_TEXTURES_IN_USE < ((map[G_LIBBOSS.TEXTURES.X]*map[G_LIBBOSS.TEXTURES.Y]) - 1) then
			ATLAS_MAP_TEXTURES_IN_USE = ((map[G_LIBBOSS.TEXTURES.X]*map[G_LIBBOSS.TEXTURES.Y]) - 1)
		end	
		--
		-- Add map pins for bosses
		--
		for i = 0, ATLAS_MAP_PINS_IN_USE - 1 do
			hideAtlasMapPin(i)			
		end

		local mapPinsMade = 0
				
		for _,v in ipairs(LB:returnBossList(zone,LB:determineZoneType(zone,ATLAS.isVeteranDifficulty),level)) do
			if v[G_LIBBOSS.INDEX.X] == 0 and v[G_LIBBOSS.INDEX.Y] == 0 then
				-- do nothing
			else
				createAtlasMapPins(ATLAS_Map.bd, mapPinsMade, v[G_LIBBOSS.INDEX.NAME], v[G_LIBBOSS.INDEX.X]*x, v[G_LIBBOSS.INDEX.Y]*y)
				mapPinsMade = mapPinsMade + 1
			end
		end
			
		if ATLAS_MAP_PINS_IN_USE < mapPinsMade then
			ATLAS_MAP_PINS_IN_USE = mapPinsMade
		end
	else
		--
		-- hide everything
		--				
		showAtlasWindows(false)		
	end	
	--
	-- Adjust Ui related to zone
	--

	if data then
		local titleText = zone .. " ["..data[G_LIBBOSS.ZONE_DATA.ZONE_MINLVL].."-"..data[G_LIBBOSS.ZONE_DATA.ZONE_MAXLVL].."]"
	
		ATLAS_Map.MapTitle:SetText(titleText)

		if atlasDungeonType == G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_NORMAL then
			ATLAS_Map.MapTitleIcon:SetTexture("/esoui/art/icons/poi/poi_groupinstance_complete.dds")
		elseif atlasDungeonType == G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_VETERAN then
			ATLAS_Map.MapTitleIcon:SetTexture("/esoui/art/icons/poi/poi_groupinstance_complete.dds")
		elseif atlasDungeonType == G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_TRIAL then
			ATLAS_Map.MapTitleIcon:SetTexture("/esoui/art/icons/poi/poi_raiddungeon_complete.dds")
		end
		if data[G_LIBBOSS.ZONE_DATA.ZONE_FACTION] == "E" then
			ATLAS_TLW.factionTextureLeft:SetTexture("/esoui/art/campaign/overview_scoringbg_ebonheart_left.dds")
			ATLAS_TLW.factionTextureRight:SetTexture("/esoui/art/campaign/overview_scoringbg_ebonheart_right.dds")
			ATLAS_LootWindow.iconTexture:SetTexture("/esoui/art/guild/banner_ebonheart.dds")			
			ATLAS_LootWindow.iconTexture:SetHidden(false)
			
			ATLAS_TLW.FactionHeaderIconTexture:SetTexture("/esoui/art/compass/ava_borderkeep_pin_ebonheart.dds")			
			ATLAS_TLW.FactionHeaderIconTexture:SetHidden(false)				
		elseif data[G_LIBBOSS.ZONE_DATA.ZONE_FACTION] == "D" then
			ATLAS_TLW.factionTextureLeft:SetTexture("/esoui/art/campaign/overview_scoringbg_daggerfall_left.dds")
			ATLAS_TLW.factionTextureRight:SetTexture("/esoui/art/campaign/overview_scoringbg_daggerfall_right.dds")
			ATLAS_LootWindow.iconTexture:SetTexture("/esoui/art/guild/banner_daggerfall.dds")
			ATLAS_LootWindow.iconTexture:SetHidden(false)
			ATLAS_TLW.FactionHeaderIconTexture:SetTexture("/esoui/art/compass/ava_borderkeep_pin_daggerfall.dds")			
			ATLAS_TLW.FactionHeaderIconTexture:SetHidden(false)				
		elseif data[G_LIBBOSS.ZONE_DATA.ZONE_FACTION] == "A" then
			ATLAS_TLW.factionTextureLeft:SetTexture("/esoui/art/campaign/overview_scoringbg_aldmeri_left.dds")
			ATLAS_TLW.factionTextureRight:SetTexture("/esoui/art/campaign/overview_scoringbg_aldmeri_right.dds")
			ATLAS_LootWindow.iconTexture:SetTexture("/esoui/art/guild/banner_aldmeri.dds")
			ATLAS_LootWindow.iconTexture:SetHidden(false)
			ATLAS_TLW.FactionHeaderIconTexture:SetTexture("/esoui/art/compass/ava_borderkeep_pin_aldmeri.dds")			
			ATLAS_TLW.FactionHeaderIconTexture:SetHidden(false)				
		else
			ATLAS_TLW.factionTextureLeft:SetTexture("Atlas/art/background_common_left.dds")
			ATLAS_TLW.factionTextureRight:SetTexture("Atlas/art/background_common_right.dds")							
			
			ATLAS_LootWindow.iconTexture:SetHidden(true)
			
			ATLAS_TLW.FactionHeaderIconTexture:SetHidden(true)		
		end	
			
		for i = 1, ATLAS_MAX_DUNGEON_LEVELS do			
			if LB:getMapTextures(zone, atlasDungeonType, i) then
				ATLAS_Map.level[i]:SetHidden(false)
			else
				ATLAS_Map.level[i]:SetHidden(true)
			end
		end
	end
	ZO_WorldMap:SetHidden( true )	
end

local function loadRightPaneZones(atlasDungeonType)
	local dataSource = determineDataSource(atlasDungeonType)

	if not dataSource then return end

	if atlasDungeonType == G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_TRIAL or 
	   atlasDungeonType == G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_VETERAN then	
		ATLAS.isVeteranDifficulty = true 
	else 
		ATLAS.isVeteranDifficulty = false
	end

	ATLAS.lastDungeonTypeRequested = atlasDungeonType
	--
	-- Add data to the scrollList
	--
	local scrollData = ZO_ScrollList_GetDataList(ATLAS_RIGHTPANE.ScrollList)

	ZO_ClearNumericallyIndexedTable(scrollData)

	for _, v in pairs(dataSource) do
		if  v[G_LIBBOSS.ZONE_DATA.ZONE_FACTION] == DAGGERFALL then
			factionTexturePath = "/esoui/art/compass/ava_borderkeep_pin_daggerfall.dds"
		elseif v[G_LIBBOSS.ZONE_DATA.ZONE_FACTION] == ALDMERI then
			factionTexturePath = "/esoui/art/compass/ava_borderkeep_pin_aldmeri.dds"
		elseif v[G_LIBBOSS.ZONE_DATA.ZONE_FACTION] == EBONHEART then
			factionTexturePath = "/esoui/art/compass/ava_borderkeep_pin_ebonheart.dds"
		elseif v[G_LIBBOSS.ZONE_DATA.ZONE_FACTION] == UNKNOWN then
			factionTexturePath = "Atlas/art/ava_borderkeep_pin_common.dds"
		end

		table.insert(scrollData, ZO_ScrollList_CreateDataEntry(ATLAS_LOCATION_DATA, 
			{
				locationName = v[G_LIBBOSS.ZONE_DATA.ZONE_EN], 
				NormalLevel = v[G_LIBBOSS.ZONE_DATA.ZONE_MINLVL].."-"..v[G_LIBBOSS.ZONE_DATA.ZONE_MAXLVL],				
				factionIcon = factionTexturePath
			}
		) )   
	end 

	ZO_ScrollList_Commit(ATLAS_RIGHTPANE.ScrollList)	
end

local function createAtlasRIghtPane()
	--
	-- Steal dimensions and anchor from similar object (location list)
	--
	local x,y = ZO_WorldMapLocations:GetDimensions()
	local isValidAnchor, point, relativeTo, relativePoint, offsetX, offsetY = ZO_WorldMapLocations:GetAnchor()
	--
	-- Define the toplevelwindow to hold the scrollList
	--
	ATLAS_RIGHTPANE = WINDOW_MANAGER:CreateTopLevelWindow(ATLAS.addonName.."TLW".."RIGHTPANE")
	ATLAS_RIGHTPANE:SetMouseEnabled(true)		
	ATLAS_RIGHTPANE:SetMovable( false )
	ATLAS_RIGHTPANE:SetClampedToScreen(true)
	ATLAS_RIGHTPANE:SetDimensions( x, y )
	ATLAS_RIGHTPANE:SetAnchor( point, relativeTo, relativePoint, offsetX, offsetY )
	ATLAS_RIGHTPANE:SetHidden( true )
	--
	-- Create Instance Type Selection headers
	--
	ATLAS_RIGHTPANE.InstanceType =  WINDOW_MANAGER:CreateControl("$(parent)InstanceType",ATLAS_RIGHTPANE,nil) 
	ATLAS_RIGHTPANE.InstanceType:SetAnchor( TOPLEFT, ATLAS_RIGHTPANE, TOPLEFT, 0, 0 )
	ATLAS_RIGHTPANE.InstanceType:SetHeight(48)
	--
	-- Add texture for normal group dungeons
	--
	ATLAS_RIGHTPANE.InstanceType.NormalGroup = WINDOW_MANAGER:CreateControl(nil, ATLAS_RIGHTPANE.InstanceType, CT_TEXTURE)
  	ATLAS_RIGHTPANE.InstanceType.NormalGroup:SetDimensions(48,48)
  	ATLAS_RIGHTPANE.InstanceType.NormalGroup:SetAnchor(TOPLEFT, ATLAS_RIGHTPANE.InstanceType, TOPLEFT, 40, 0)  
  	ATLAS_RIGHTPANE.InstanceType.NormalGroup:SetTexture("/esoui/art/icons/poi/poi_groupinstance_complete.dds")  	

  	ATLAS_RIGHTPANE.InstanceType.NormalGroup.Icon = WINDOW_MANAGER:CreateControl(nil, ATLAS_RIGHTPANE.InstanceType.NormalGroup, CT_TEXTURE)
  	ATLAS_RIGHTPANE.InstanceType.NormalGroup.Icon:SetDimensions(32,32)
  	ATLAS_RIGHTPANE.InstanceType.NormalGroup.Icon:SetAnchor(CENTER, ATLAS_RIGHTPANE.InstanceType.NormalGroup, CENTER, 0, 14)  
  	ATLAS_RIGHTPANE.InstanceType.NormalGroup.Icon:SetTexture("/EsoUI/Art/LFG/LFG_normalDungeon_down.dds")
  	ATLAS_RIGHTPANE.InstanceType.NormalGroup.Icon:SetBlendMode(ADD)

  	ATLAS_RIGHTPANE.InstanceType.NormalGroup:SetHandler("OnMouseUp", function() loadRightPaneZones(G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_NORMAL) end)
  	AtlasToolTipCreator(ATLAS_RIGHTPANE.InstanceType.NormalGroup,"Normal Group Instances")  	
	--
	-- Add texture for veteran group dungeons
	--
	ATLAS_RIGHTPANE.InstanceType.VeteranGroup = WINDOW_MANAGER:CreateControl(nil, ATLAS_RIGHTPANE.InstanceType, CT_TEXTURE)
  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup:SetDimensions(48,48)
  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup:SetAnchor(LEFT, ATLAS_RIGHTPANE.InstanceType.NormalGroup, RIGHT, 20, 0)  
  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup:SetTexture("/esoui/art/icons/poi/poi_groupinstance_complete.dds")  	

  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup.Icon = WINDOW_MANAGER:CreateControl(nil, ATLAS_RIGHTPANE.InstanceType.VeteranGroup, CT_TEXTURE)
  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup.Icon:SetDimensions(32,32)
  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup.Icon:SetAnchor(CENTER, ATLAS_RIGHTPANE.InstanceType.VeteranGroup, CENTER, 0, 14)  
  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup.Icon:SetTexture("/EsoUI/Art/LFG/LFG_veteranDungeon_down.dds")
  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup.Icon:SetBlendMode(ADD)

  	ATLAS_RIGHTPANE.InstanceType.VeteranGroup:SetHandler("OnMouseUp", function() loadRightPaneZones(G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_VETERAN) end)
  	AtlasToolTipCreator(ATLAS_RIGHTPANE.InstanceType.VeteranGroup,"Veteran Group Instances")  	  	
  	--
	-- Add texture for trials dungeons
	--
	ATLAS_RIGHTPANE.InstanceType.Trial = WINDOW_MANAGER:CreateControl(nil, ATLAS_RIGHTPANE.InstanceType, CT_TEXTURE)
  	ATLAS_RIGHTPANE.InstanceType.Trial:SetDimensions(48,48)
  	ATLAS_RIGHTPANE.InstanceType.Trial:SetAnchor(LEFT, ATLAS_RIGHTPANE.InstanceType.VeteranGroup, RIGHT, 20, 0)  
  	ATLAS_RIGHTPANE.InstanceType.Trial:SetTexture("/esoui/art/icons/poi/poi_raiddungeon_complete.dds")
  	ATLAS_RIGHTPANE.InstanceType.Trial:SetHandler("OnMouseUp", function() loadRightPaneZones(G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_TRIAL) end)
  	
  	AtlasToolTipCreator(ATLAS_RIGHTPANE.InstanceType.Trial,"Trial Instances")  	
	--
	-- Create Sort Headers
	--
	ATLAS_RIGHTPANE.Headers = WINDOW_MANAGER:CreateControl("$(parent)Headers",ATLAS_RIGHTPANE,nil) 
	ATLAS_RIGHTPANE.Headers:SetAnchor( TOPLEFT, ATLAS_RIGHTPANE.InstanceType, BOTTOMLEFT, 0, 0 )
	ATLAS_RIGHTPANE.Headers:SetHeight(32)

	ATLAS_RIGHTPANE.Headers.Faction = WINDOW_MANAGER:CreateControlFromVirtual("$(parent)Faction",ATLAS_RIGHTPANE.Headers,"ZO_SortHeaderIcon") 
	ATLAS_RIGHTPANE.Headers.Faction:SetDimensions(16,32)
	ATLAS_RIGHTPANE.Headers.Faction:SetAnchor( TOPLEFT, ATLAS_RIGHTPANE.Headers, TOPLEFT, 8, 0 )
	ZO_SortHeader_InitializeArrowHeader(ATLAS_RIGHTPANE.Headers.Faction, "factionIcon", ZO_SORT_ORDER_UP)
	ZO_SortHeader_SetTooltip(ATLAS_RIGHTPANE.Headers.Faction, "Sort on Faction")

	ATLAS_RIGHTPANE.Headers.Dungeon = WINDOW_MANAGER:CreateControlFromVirtual("$(parent)Dungeon",ATLAS_RIGHTPANE.Headers,"ZO_SortHeader") 
	ATLAS_RIGHTPANE.Headers.Dungeon:SetDimensions(160,32)
	ATLAS_RIGHTPANE.Headers.Dungeon:SetAnchor( LEFT, ATLAS_RIGHTPANE.Headers.Faction, RIGHT, 18, 0 )
	ZO_SortHeader_Initialize(ATLAS_RIGHTPANE.Headers.Dungeon, "Dungeon List", "locationName", ZO_SORT_ORDER_UP, TEXT_ALIGN_LEFT, "ZoFontGameLargeBold")
	ZO_SortHeader_SetTooltip(ATLAS_RIGHTPANE.Headers.Dungeon, "Sort on Dungeons")

	ATLAS_RIGHTPANE.Headers.NormalLevel = WINDOW_MANAGER:CreateControlFromVirtual("$(parent)NormalLevel",ATLAS_RIGHTPANE.Headers,"ZO_SortHeaderIcon") 
	ATLAS_RIGHTPANE.Headers.NormalLevel:SetDimensions(16,32)
	ATLAS_RIGHTPANE.Headers.NormalLevel:SetAnchor( LEFT, ATLAS_RIGHTPANE.Headers.Dungeon, RIGHT, 22, 0 )
	ZO_SortHeader_InitializeArrowHeader(ATLAS_RIGHTPANE.Headers.NormalLevel, "NormalLevel", ZO_SORT_ORDER_UP)
	ZO_SortHeader_SetTooltip(ATLAS_RIGHTPANE.Headers.NormalLevel, "Sort on normal level")

    local sortHeaders = ZO_SortHeaderGroup:New(ATLAS_RIGHTPANE:GetNamedChild("Headers"), SHOW_ARROWS)
    sortHeaders:RegisterCallback(
    	ZO_SortHeaderGroup.HEADER_CLICKED, 
    	function(key, order)
    		table.sort(
    			ZO_ScrollList_GetDataList(ATLAS_RIGHTPANE.ScrollList), 
    			function(entry1, entry2) 
    				return ZO_TableOrderingFunction(entry1.data, entry2.data, key, ATLAS_ScrollList_SORT_KEYS, order)
    			end)

        	ZO_ScrollList_Commit(ATLAS_RIGHTPANE.ScrollList)
    	end)
    sortHeaders:AddHeadersFromContainer()
    --sortHeaders:SelectHeaderByKey("Dungeon", ZO_SortHeaderGroup.SUPPRESS_CALLBACKS)

	--
	-- Create the scrollList, but don't fill it yet.
	--	
	ATLAS_RIGHTPANE.ScrollList = WINDOW_MANAGER:CreateControlFromVirtual("$(parent)ScrollList", ATLAS_RIGHTPANE, "ZO_ScrollList")	
	ATLAS_RIGHTPANE.ScrollList:SetDimensions(x, y-32)
	ATLAS_RIGHTPANE.ScrollList:SetAnchor(TOPLEFT, ATLAS_RIGHTPANE.Headers, BOTTOMLEFT, 0, 0)
	--
	-- Add a datatype to the scrollList
	--
	ZO_ScrollList_Initialize(ATLAS_RIGHTPANE.ScrollList)
	ZO_ScrollList_EnableHighlight(ATLAS_RIGHTPANE.ScrollList, "ZO_ThinListHighlight")
	ZO_ScrollList_AddDataType(ATLAS_RIGHTPANE.ScrollList, ATLAS_LOCATION_DATA, "AtlasLocationRow", 23, 
		function(control, data) 				
			control:GetNamedChild("Location"):SetText(data.locationName) 
			control:GetNamedChild("NormalLevel"):SetText(data.NormalLevel) 
			--control:GetNamedChild("VeteranLevel"):SetText(data.VeteranLevel) 
			control:GetNamedChild("Faction"):SetTexture(data.factionIcon)
		end
	)
	
	loadRightPaneZones(G_LIBBOSS.ZONE_TYPE.DUNGEON_INSTANCED_NORMAL)	
	--
	-- Create button for adding to the Rightpane
	--
	local buttonData = 
	{ 
		--descriptor = "AtlasRightPaneButton",
	  	normal = "EsoUI/Art/mainmenu/menubar_journal_up.dds",
        pressed = "EsoUI/Art/mainmenu/menubar_journal_down.dds",
        --disabled = "EsoUI/Art/mainmenu/menubar_journal_disabled.dds",
        highlight = "EsoUI/Art/mainmenu/menubar_journal_over.dds",                  
    }
    --
	-- Create a fragment from the window and add it to the modeBar of the WorldMap RightPane
	--
	local bossFragment = ZO_FadeSceneFragment:New(ATLAS_RIGHTPANE) 
    WORLD_MAP_INFO.modeBar:Add(ATLAS_LOOT_NAME, {bossFragment}, buttonData)
    --
    -- Create a few preHookHandlers so that the map shows up back again when hidden
    --     
    ZO_PreHookHandler(ATLAS_RIGHTPANE     , "OnHide", function() showAtlasWindows( false ) end)
    ZO_PreHookHandler(ZO_WorldMapLocations, "OnShow", function() ZO_WorldMap:SetHidden( false ) showAtlasWindows( false ) end)
    ZO_PreHookHandler(ZO_WorldMapFilters  , "OnShow", function() ZO_WorldMap:SetHidden( false ) showAtlasWindows( false ) end)
    ZO_PreHookHandler(ZO_WorldMapKey      , "OnShow", function() ZO_WorldMap:SetHidden( false ) showAtlasWindows( false ) end)
    ZO_PreHookHandler(ZO_WorldMapQuests   , "OnShow", function() ZO_WorldMap:SetHidden( false ) showAtlasWindows( false ) end)	
    ZO_PreHookHandler(ATLAS_RIGHTPANE     , "OnShow", function() ATLAS.chatWasMinimized = CHAT_SYSTEM:IsMinimized() buildAtlasMap(GetMapName()) showAtlasLoot(GetMapName()) end)
end

local function createAtlasInterface()
	--
	-- This object should hold everything ...
	--
	local x,y = ZO_WorldMap:GetDimensions()
	x = x + 500
	y = y + 80

	ATLAS_TLW = WINDOW_MANAGER:CreateTopLevelWindow(nil)	
	ATLAS_TLW:SetMovable( false )
	ATLAS_TLW:SetClampedToScreen(true)
	ATLAS_TLW:SetDimensions( x , y )
	ATLAS_TLW:SetAnchor( TOPRIGHT, ZO_WorldMap, TOPRIGHT, 40, -40 )
	ATLAS_TLW:SetHidden( true )
	--
	-- Define a label that holds the addonName and show it on top of the TLW
	--
	ATLAS_TLW.title = WINDOW_MANAGER:CreateControl(nil, ATLAS_TLW, CT_LABEL)
	ATLAS_TLW.title:SetColor(0.8, 0.8, 0.8, 1)
	ATLAS_TLW.title:SetFont("ZoFontAlert")
	ATLAS_TLW.title:SetScale(1.5)
	ATLAS_TLW.title:SetWrapMode(TEX_MODE_CLAMP)
	ATLAS_TLW.title:SetDrawLayer(2)
	ATLAS_TLW.title:SetText("Atlas")
	ATLAS_TLW.title:SetAnchor(TOP, ATLAS_TLW, nil, 110, -10)
	ATLAS_TLW.title:SetDimensions(200,25)	
  	--
  	-- Define a texture that holds the faction of the dungeon
  	--
  	ATLAS_TLW.factionTextureLeft = WINDOW_MANAGER:CreateControl(nil, ATLAS_TLW, CT_TEXTURE)
  	ATLAS_TLW.factionTextureLeft:SetDimensions(x-240,y-500)
  	ATLAS_TLW.factionTextureLeft:SetAnchor(TOPLEFT, ATLAS_TLW, TOPLEFT, 40, 0)  	
	
  	ATLAS_TLW.factionTextureRight = WINDOW_MANAGER:CreateControl(nil, ATLAS_TLW, CT_TEXTURE)
  	ATLAS_TLW.factionTextureRight:SetDimensions(400,y-500)
  	ATLAS_TLW.factionTextureRight:SetAnchor(TOPRIGHT, ATLAS_TLW, TOPRIGHT, 200, 0)  
  	--
	-- Define a divider above the faction textures
	--
	ATLAS_TLW.titledividerLeft = WINDOW_MANAGER:CreateControl(nil,  ATLAS_TLW, CT_TEXTURE)
  	ATLAS_TLW.titledividerLeft:SetDimensions(x-100,4)
  	ATLAS_TLW.titledividerLeft:SetAnchor(TOPLEFT, ATLAS_TLW, TOPLEFT, 50, 6)
  	ATLAS_TLW.titledividerLeft:SetTexture("/esoui/art/guild/sectiondivider_left.dds")	

  	ATLAS_TLW.titledividerRight = WINDOW_MANAGER:CreateControl(nil,  ATLAS_TLW, CT_TEXTURE)
  	ATLAS_TLW.titledividerRight:SetDimensions(100,4)
  	ATLAS_TLW.titledividerRight:SetAnchor(TOPLEFT, ATLAS_TLW.titledividerLeft, TOPRIGHT, 0, 0)
  	ATLAS_TLW.titledividerRight:SetTextureCoords(0, 1, 0, 0.391)
  	ATLAS_TLW.titledividerRight:SetTexture("/esoui/art/guild/sectiondivider_right.dds")
  	--
  	-- Add a faction texture
  	--
  	ATLAS_TLW.FactionHeaderIconTexture = WINDOW_MANAGER:CreateControl(nil,  ATLAS_TLW, CT_TEXTURE)
    ATLAS_TLW.FactionHeaderIconTexture:SetDimensions(120,120)
    ATLAS_TLW.FactionHeaderIconTexture:SetAnchor(TOP, ATLAS_TLW, nil, (x/2)-140, 50)
    --ATLAS_TLW.FactionHeaderIconTexture:SetTexture("")	--Will be set when zone gets determined
	--
	-- Define a ToplevelWindow that holds the map textures
	--
	local x,y = ZO_WorldMap:GetDimensions()	
	x = x*ATLAS_MAP_SCALE
	y = y*ATLAS_MAP_SCALE

	ATLAS_Map = WINDOW_MANAGER:CreateTopLevelWindow("ATLAS_MAP")
	ATLAS_Map:SetMouseEnabled(true)		
	ATLAS_Map:SetMovable( false )
	ATLAS_Map:SetClampedToScreen(true)
	ATLAS_Map:SetDimensions( x , y )
	ATLAS_Map:SetAnchor( BOTTOMRIGHT, ATLAS_TLW, BOTTOMRIGHT, -100, -40 )
	ATLAS_Map:SetHidden( true )	
	--
  	-- Adding mapTitleIcon to indicate what kind of dungeon we are looking at
  	--
  	ATLAS_Map.MapTitleIcon = WINDOW_MANAGER:CreateControl(nil,  ATLAS_Map, CT_TEXTURE)
    ATLAS_Map.MapTitleIcon:SetDimensions(35,35)
    ATLAS_Map.MapTitleIcon:SetAnchor(TOP, ATLAS_Map, nil, (-x/2)+18, -35)
    -- ATLAS_Map.MapTitleIcon:SetTexture("/esoui/art/icons/poi/poi_groupinstance_complete.dds") -- Will be set when zone gets determined  
    --
    -- Adding mapTitleText that holds the dungeon name
    --
  	ATLAS_Map.MapTitle = WINDOW_MANAGER:CreateControl(nil, ATLAS_Map, CT_LABEL)
    ATLAS_Map.MapTitle:SetColor(0.8, 0.8, 0.8, 1)
    ATLAS_Map.MapTitle:SetFont("ZoFontAlert")
    --ATLAS_Map.MapTitle:SetText("") -- Will be set when zone gets determined
    ATLAS_Map.MapTitle:SetAnchor(LEFT, ATLAS_Map.MapTitleIcon, RIGHT, 0, 0)
    ATLAS_Map.MapTitle:SetDimensions(400,25)    
	--
	-- Define the Frame that goes around the map
	-- 
	ATLAS_Map.bd = WINDOW_MANAGER:CreateControl(nil, ATLAS_Map, CT_BACKDROP)	
	ATLAS_Map.bd:SetAnchorFill(ATLAS_Map)
	ATLAS_Map.bd:SetCenterColor(0,0,0,0)
	ATLAS_Map.bd:SetEdgeTexture("EsoUI/Art/WorldMap/worldmap_frame_edge.dds",128,16,0,0)
	ATLAS_Map.bd:SetInsets(16,16,-16,-16)	
	ATLAS_Map.bd:SetDrawLayer(1)
	--
	-- Make the frame dirty
	--
	ATLAS_Map.bd.texturetop = WINDOW_MANAGER:CreateControlFromVirtual(nil, ATLAS_Map, "ZO_WorldMapFrameMunge")
	ATLAS_Map.bd.texturetop:SetAnchor( TOPLEFT, ATLAS_Map.bd, TOPLEFT, 4, 0 )
	ATLAS_Map.bd.texturetop:SetDimensions( x-8, 2)	
	
	ATLAS_Map.bd.textureleft = WINDOW_MANAGER:CreateControlFromVirtual(nil, ATLAS_Map, "ZO_WorldMapFrameMunge")
	ATLAS_Map.bd.textureleft:SetAnchor( TOPLEFT, ATLAS_Map.bd, TOPLEFT, 0, 4 )
	ATLAS_Map.bd.textureleft:SetDimensions( 2, y-8)

	ATLAS_Map.bd.textureright = WINDOW_MANAGER:CreateControlFromVirtual(nil, ATLAS_Map, "ZO_WorldMapFrameMunge")
	ATLAS_Map.bd.textureright:SetAnchor( TOPRIGHT, ATLAS_Map.bd, TOPRIGHT, 0, 4 )
	ATLAS_Map.bd.textureright:SetDimensions( 2, y-8)

	ATLAS_Map.bd.texturebottom = WINDOW_MANAGER:CreateControlFromVirtual(nil, ATLAS_Map, "ZO_WorldMapFrameMunge")
	ATLAS_Map.bd.texturebottom:SetAnchor( BOTTOMLEFT, ATLAS_Map.bd, BOTTOMLEFT, 4, 0 )
	ATLAS_Map.bd.texturebottom:SetDimensions( x-8, 2)	
	--
	-- Texture for showing player position on map
	--
	ATLAS_Map.playerIcon = WINDOW_MANAGER:CreateControl(nil,  ATLAS_Map.bd, CT_TEXTURE)
	ATLAS_Map.playerIcon:SetDimensions(16,16)
	ATLAS_Map.playerIcon:SetAnchor(CENTER, ATLAS_Map.bd, TOPLEFT, 0, 0)
	ATLAS_Map.playerIcon:SetHidden(false)
	ATLAS_Map.playerIcon:SetTexture("/esoui/art/mappins/group_pin.dds")	
	--ATLAS_Map.playerIcon:SetTexture("/esoui/art/mappins/ui-worldmapplayerpip.dds")	
	ATLAS_Map.playerIcon:SetDrawLayer(2)
	--
	-- Attach to the map frame the dungeon level indicators
	--
	ATLAS_Map.level = {}
	for i = 1, ATLAS_MAX_DUNGEON_LEVELS do		
		ATLAS_Map.level[i] = WINDOW_MANAGER:CreateControl(nil,  ATLAS_Map, CT_LABEL)
  		ATLAS_Map.level[i]:SetDimensions(32,32)
  		ATLAS_Map.level[i]:SetColor(0.8, 0.8, 0.8, 1)
    	ATLAS_Map.level[i]:SetFont("ZoFontAlert")
  		ATLAS_Map.level[i]:SetText(i)
  		ATLAS_Map.level[i]:SetMouseEnabled(true)
  		ATLAS_Map.level[i]:SetHandler("OnMouseUp", function() buildAtlasMap(ATLAS.lastZoneRequested, ATLAS.lastDungeonTypeRequested, i) end)
  		if i == 1 then
  			ATLAS_Map.level[i]:SetAnchor(TOPLEFT, ATLAS_Map, TOPRIGHT, 10, 0)  	  	
  		else
  			ATLAS_Map.level[i]:SetAnchor(TOP, ATLAS_Map.level[i-1], BOTTOM, 0, 0)  	  	
  		end
  		AtlasToolTipCreator(ATLAS_Map.level[i], "Dungeon level "..i)
  	end
	--
    -- Define a ToplevelWindow to attach to the WorldMap that holds the loot
    --
    local x,y = ATLAS_Map:GetDimensions()	

    ATLAS_LootWindow = WINDOW_MANAGER:CreateTopLevelWindow(nil)
	ATLAS_LootWindow:SetMouseEnabled(true)		
	ATLAS_LootWindow:SetMovable( false )
	ATLAS_LootWindow:SetClampedToScreen(true)
	ATLAS_LootWindow:SetDimensions( 400 , y )
	ATLAS_LootWindow:SetAnchor( TOPRIGHT, ATLAS_Map, TOPLEFT, -20, 0 )
	ATLAS_LootWindow:SetHidden( true )	
	--
	-- Add faction icon texture to the background of the lootlist
	--
	ATLAS_LootWindow.iconTexture = WINDOW_MANAGER:CreateControl(nil,  ATLAS_LootWindow, CT_TEXTURE)
  	ATLAS_LootWindow.iconTexture:SetDimensions(400,y)
  	ATLAS_LootWindow.iconTexture:SetAnchor(CENTER, ATLAS_LootWindow, CENTER, 40, 160)  	
  	---ATLAS_LootWindow.iconTexture:SetDrawLayer(1) 	
  	--
  	-- Add the Lootwindow backdrop
  	--
  	ATLAS_LootWindow.bd = WINDOW_MANAGER:CreateControl(nil, ATLAS_LootWindow, CT_BACKDROP)
	ATLAS_LootWindow.bd:SetCenterTexture([[/esoui/art/chatwindow/chat_bg_center.dds]], 16, 1)
	ATLAS_LootWindow.bd:SetEdgeTexture([[/esoui/art/chatwindow/chat_bg_edge.dds]], 32, 32, 32, 0)
	ATLAS_LootWindow.bd:SetInsets(32,32,-32,-32)	
	ATLAS_LootWindow.bd:SetAnchorFill(ATLAS_LootWindow)
	--
	-- Add the lootwindow scrolllist that holds bosses and drops
	--
  	local x,y = ATLAS_LootWindow:GetDimensions()
  	ATLAS_LootWindow.scrollList = WINDOW_MANAGER:CreateControlFromVirtual("$(parent)ScrollList", ATLAS_LootWindow, "ZO_ScrollList")
	ATLAS_LootWindow.scrollList:SetAnchor( TOPLEFT, ATLAS_LootWindow, TOPLEFT, 20, 40 )
	ATLAS_LootWindow.scrollList:SetDimensions(x-40,y-80)
	--
	-- Add a datatype to the scrollList
	--
	ZO_ScrollList_Initialize(ATLAS_LootWindow.scrollList)
	ZO_ScrollList_EnableHighlight(ATLAS_LootWindow.scrollList, "ZO_ThinListHighlight")
	ZO_ScrollList_AddDataType(ATLAS_LootWindow.scrollList, ATLAS_DROPS_BOSS_DATA, "AtlasDropsLevel1BossLabel", 23, 
		function(control, data) 				
			local bossLabel = control:GetNamedChild("Label")

   			bossLabel:SetText(data.bossName) 
			bossLabel:SetEnabled( true )
		end
	)
	ZO_ScrollList_AddDataType(ATLAS_LootWindow.scrollList, ATLAS_LEVEL2_LABEL, "AtlasDropsLevel2Label", 23, 
		function(control, data) 				
			local Label = control:GetNamedChild("Label")

   			Label:SetText(data.Label) 
			Label:SetEnabled( true )
		end
	)	
	ZO_ScrollList_AddDataType(ATLAS_LootWindow.scrollList, ATLAS_DROPS_DROP_DATA, "AtlasDropsDropLocationRow", 23, 
		function(control, data) 				
			local Label = control:GetNamedChild("ItemLink")

   			Label:SetText(data.ItemLink) 
			Label:SetEnabled( true )
		end
	)	
	ZO_ScrollList_AddDataType(ATLAS_LootWindow.scrollList, ATLAS_DROPS_SET_ITEM_DATA, "AtlasDropsSetItemLocationRow", 23, 
		function(control, data) 				
			local Label = control:GetNamedChild("ItemLink")

   			Label:SetText(data.ItemLink) 
			Label:SetEnabled( true )
		end
	)	

	createAtlasRIghtPane()
	
end

function ATLAS_LocationRowLocation_OnMouseUp(self, button, upInside)
	ZO_WorldMap:SetHidden( true )
	showAtlasWindows( true )	
	buildAtlasMap(self:GetText(), ATLAS.lastDungeonTypeRequested)	
	showAtlasLoot(self:GetText())	
	if not ATLAS.chatWasMinimized then	
		CHAT_SYSTEM:Minimize()
	end
end

local function processSlashCommands(option)	
	local options = {}
	local searchResult = { string.match(option,"^(%S*)%s*(.-)$") }
	for i,v in pairs(searchResult) do
        	if (v ~= nil and v ~= "") then
			options[i] = string.lower(v)
        	end
	end
    	if options[1] ~= "donate" then
    		d("atlas commands are : ")
	    	d("For giving a donation : /atlas donate <value>")
    		return
	end

--	if options[1] == "loc" and not options[2] then
--    		d("Invalid command, /atlas loc <bossname>")
--	elseif options[1] == "loc" and options[2] then
--		SetMapToPlayerLocation()
--		x,y, _ = GetMapPlayerPosition("player")
--		RequestOpenMailbox()		
--		SendMail("@CrazyDutchGuy", "Update location for "..options[2],  GetMapName() .. "::" .. GetMapTileTexture() .."::" .. "["..x..","..y.."]")	
--		d("Send mail to addon author for updating location of "..options[2])
--	end
	if options[1] == "donate" and not options[2] then
    	d("Invalid command, /atlas donate <value>")
	elseif options[1] == "donate" and options[2] then
		RequestOpenMailbox()	
		QueueMoneyAttachment(options[2])	
		SendMail("@CrazyDutchGuy", "Atlas Donation")	
	end
--	if options[1] == "item" and not options[2] then
--    	d("Invalid command, /atlas item <bossname> <itemlink>")
--	elseif options[1] == "item" and options[2] then
--
--		RequestOpenMailbox()		
--		SendMail("@CrazyDutchGuy", "Update loot for Atlas", GetMapName() .. "::" .. GetMapTileTexture() .."::" .. options[2] )	
--		d("Send mail to addon author for updating ".. options[2] )
--	end
end

function ATLAS:ON_MAP_UPDATE()
	if not ATLAS_Map:IsHidden() then
		local pX, pY = GetMapPlayerPosition("player")
		local mX, mY = ATLAS_Map:GetDimensions()	
	
		ATLAS_Map.playerIcon:SetAnchor(CENTER, ATLAS_Map.bd, TOPLEFT, pX*mX, pY*mY)
	end
end                    

function ATLAS:OnLinkClicked(rawLink, mouseButton, linkText, color, linkType, ...)  
	if linkType == "set" then
		showAtlasLoot(ATLAS.lastZoneRequested, rawLink)
		return true
	end
end

function ATLAS_OnLinkMouseExit()	
end

function ATLAS_OnLinkMouseEnter(rawLink, button, upInside, ctrl, alt, shift)
	ZO_LinkHandler_OnLinkClicked(rawLink, 1)
	if ctrl then
		ZO_LinkHandler_InsertLink(zo_strformat(rawLink))
	end
end

function ATLAS:EVENT_ADD_ON_LOADED(eventCode, addonName, ...)
	if addonName == ATLAS.addonName then	
		ATLAS.SavedVariables = ZO_SavedVars:New("Atlas_SavedVariables", 2, nil, ATLAS.defaults)	
		createAtlasInterface()		

 		
		SLASH_COMMANDS["/atlas"] = processSlashCommands
		--		
		-- Unregister events we are not using anymore
		--
		EVENT_MANAGER:UnregisterForEvent( ATLAS.addonName, EVENT_ADD_ON_LOADED )

		WORLD_MAP_SCENE:RegisterCallback("StateChange", function(oldState, newState)
        	if newState == SCENE_SHOWING then            
            	EVENT_MANAGER:RegisterForUpdate(ATLAS.addonName.."MapUpdate", 1000, function() ATLAS:ON_MAP_UPDATE() end)
         	elseif newState == SCENE_HIDING then
            	EVENT_MANAGER:UnregisterForUpdate(ATLAS.addonName.."MapUpdate")
         	end
      	end)			

      	LINK_HANDLER:RegisterCallback(LINK_HANDLER.LINK_CLICKED_EVENT, function(...) return ATLAS:OnLinkClicked(...) end)
      	--LINK_HANDLER:RegisterCallback(LINK_HANDLER.INSERT_LINK_EVENT, function(...) ATLAS:OnLinkClicked(...) end)

	end
end

function ATLAS:EVENT_PLAYER_ACTIVATED(...)
	d("|cFF2222ATLAS|r addon Loaded, /atlas for more info")
	--
	-- Only once so unreg is from further events
	--
	EVENT_MANAGER:UnregisterForEvent( ATLAS.addonName, EVENT_PLAYER_ACTIVATED )	
end

function ATLAS_OnInitialized()
	EVENT_MANAGER:RegisterForEvent(ATLAS.addonName, EVENT_ADD_ON_LOADED, function(...) ATLAS:EVENT_ADD_ON_LOADED(...) end )		
	EVENT_MANAGER:RegisterForEvent(ATLAS.addonName, EVENT_PLAYER_ACTIVATED, function(...) ATLAS:EVENT_PLAYER_ACTIVATED(...) end)
end
